# coding: utf-8

import os

from .errors import *


class PathCombO:
    """被包装的路径类，主要是为了提供与Anaconda3路径相关的环境变量。"""

    def __init__(self, _path):
        """_path不要求是否是目录或文件路径，但最后会取其目录路径。"""
        if not isinstance(_path, str):
            raise ParamTypeError("类型错误，路径参数类型应为字符串。")
        if not os.path.exists(_path):
            raise PathParamError("参数_path值不是有效的文件或目录路径。")
        if not os.path.isdir(_path):
            _path = os.path.dirname(_path)
        self.__path = os.path.normpath(_path)

    @property
    def path(self):
        return self.__path

    @property
    def is_anaconda(self):
        _path = self.path
        while True:
            if os.path.isfile(os.path.join(_path, "_conda.exe")):
                return True
            temp_path = os.path.dirname(_path)
            if os.path.samefile(temp_path, _path):
                break
            _path = temp_path
        return False

    def __repr__(self):
        return self.__path

    @staticmethod
    def is_conda_venv(_path):
        if not os.path.isfile(os.path.join(_path, "python.exe")):
            return False
        if os.path.basename(os.path.dirname(_path)) == "envs":
            return True
        return False

    @staticmethod
    def anaconda_main(_path):
        if os.path.isfile(os.path.join(_path, "_conda.exe")):
            return _path
        if PathCombO.is_conda_venv(_path):
            return os.path.dirname(os.path.dirname(_path))
        return ""

    def environment_variable(self):
        if self.is_anaconda:
            _path = self.path
            if PathCombO.is_conda_venv(_path):
                name, shlvl = os.path.basename(_path), 2
            else:
                name, shlvl = "base", 1
            anaconda_main_path = self.anaconda_main(_path)
            preset_env_var = {
                "CONDA_DEFAULT_ENV": r"{}".format(name),
                "CONDA_EXE": os.path.join(
                    anaconda_main_path,
                    "Scripts",
                    "conda.exe",
                ),
                "CONDA_PREFIX": _path
                if PathCombO.is_conda_venv(_path)
                else anaconda_main_path,
                "CONDA_PREFIX_1": anaconda_main_path,
                "CONDA_PROMPT_MODIFIER": r"({}) ".format(name),
                "CONDA_PYTHON_EXE": os.path.join(anaconda_main_path, "python.exe"),
                "CONDA_SHLVL": r"{}".format(shlvl),
            }
            if shlvl == 1:
                del preset_env_var["CONDA_PREFIX_1"]
            preset_env_var.update(os.environ)
            conda_PATH = "{};{};{};{};{};{};{};".format(
                _path,
                os.path.join(_path, "Library", "mingw-w64", "bin"),
                os.path.join(_path, "Library", "usr", "bin"),
                os.path.join(_path, "Library", "bin"),
                os.path.join(_path, "Scripts"),
                os.path.join(_path, "bin"),
                os.path.join(anaconda_main_path, "condabin"),
            )
            preset_env_var["PATH"] = conda_PATH + preset_env_var.get("PATH", "")
            return preset_env_var
        return os.environ



# coding: utf-8

NAME = "fastpip"
version = 0, 11, 2
AUTHOR = "hrpzcf"
EMAIL = "hrpzcf@foxmail.com"
WEBSITE = "https://gitee.com/hrpzcf/fastpip"
VERSION = ".".join(map(str, version))

__all__ = ["NAME", "AUTHOR", "EMAIL", "WEBSITE", "VERSION", "version"]



# -*- coding: utf-8 -*-

import os

from psutil import disk_partitions


def __common_location():
    """生成各磁盘上的常见的Python安装目录路径列表。"""
    most_possible_path = list()
    common_dir = (
        "Program Files",
        "Program Files (x86)",
        "ProgramData",
    )
    most_possible_path.append(os.path.expanduser("~"))
    disk_parts = [dp.device for dp in disk_partitions()]
    most_possible_path.extend(disk_parts)
    for dp in disk_parts:
        for cd in common_dir:
            full_path = os.path.join(dp, cd)
            if not os.path.isdir(full_path):
                continue
            most_possible_path.append(full_path)
    appd = os.path.join(os.getenv("LOCALAPPDATA"), "Programs")
    if os.path.exists(appd):
        most_possible_path.append(appd)
    return most_possible_path


def __fsize(*_fpath):
    """返回文件路径中文件的大小。"""
    try:
        return os.path.getsize(os.path.join(*_fpath))
    except Exception:
        return False


def __paths_in_PATH():
    """
    ### 查找系统环境变量PATH中的Python目录路径列表。
    仅根据"目录中是否存在python.exe文件"进行简单查找。
    """
    python_paths_found = list()
    PATH_paths = os.getenv("PATH", "").split(";")
    for PATH_path in PATH_paths:
        try:
            PATH_path_files = os.listdir(PATH_path)
        except Exception:
            continue
        PATH_path = os.path.normpath(PATH_path)
        if (
            "python.exe" in PATH_path_files
            and __fsize(PATH_path, "python.exe")
            and PATH_path not in python_paths_found
        ):
            python_paths_found.append(PATH_path)
    return python_paths_found


def cur_py_path():
    """
    ### 返回系统环境变量PATH中第一个Python目录路径。
    如果环境变量PATH中没有Python目录路径则返回空字符串。
    """
    PATH_paths = __paths_in_PATH()
    if not PATH_paths:
        return ""
    return PATH_paths[0]


def __list_fd(_path, t="b"):
    """列出给定目录下的文件或文件夹，返回文件或文件夹列表。"""
    results = list()
    if os.path.isfile(_path):
        return results
    if t == "f":
        condi = os.path.isfile
    elif t == "d":
        condi = os.path.isdir
    else:
        condi = os.path.exists
    try:
        files_dirs = os.listdir(_path)
    except Exception:
        return results
    for item in files_dirs:
        if condi(os.path.join(_path, item)):
            results.append(item)
    return results


def __path_list(fd_name):
    """
    ### 判断指定路径是否为Python或Anaconda3目录。
    将确认为Python目录的路径或Anaconda3内Python目录路径添加到列表并返回。
    """
    fd_name = os.path.normpath(fd_name)
    python_env_paths = list()
    files = __list_fd(fd_name, "f")
    if "python.exe" in files and __fsize(fd_name, "python.exe"):
        python_env_paths.append(fd_name)
    if "_conda.exe" in files and __fsize(fd_name, "_conda.exe"):
        env_d = os.path.join(fd_name, "envs")
        if not os.path.isdir(env_d):
            return python_env_paths
        for env_p in __list_fd(env_d, "d"):
            env_p = os.path.join(env_d, env_p)
            if "python.exe" in __list_fd(env_p, "f"):
                python_env_paths.append(env_p)
    return python_env_paths


def all_py_paths():
    """
    ### 返回存在Python解释器的目录路径列表。
    只在常用安装位置深入搜索两层目录，不会过于深入。
    """
    paths_interpreter_exists = __paths_in_PATH()
    for common in __common_location():
        for level1 in __list_fd(common, "d"):
            l1f = os.path.join(common, level1)
            for _path in __path_list(l1f):
                if _path in paths_interpreter_exists:
                    continue
                paths_interpreter_exists.append(_path)
            for level2 in __list_fd(l1f, "d"):
                for _path in __path_list(os.path.join(l1f, level2)):
                    if _path in paths_interpreter_exists:
                        continue
                    paths_interpreter_exists.append(_path)
    return paths_interpreter_exists



# -*- coding: utf-8 -*-

from fastpip import PyEnv, all_py_paths, cur_py_path, index_urls

# 打印预置的几个国内PyPi镜像地址：
print(index_urls)
# {'aliyun': 'https://mirrors.aliyun.com/pypi/simple/',
# 'tencent': 'https://mirrors.cloud.tencent.com/pypi/simple',
# 'douban': 'https://pypi.doubanio.com/simple/',
# 'huawei': 'https://mirrors.huaweicloud.com/repository/pypi/simple',
# 'opentuna': 'https://opentuna.cn/pypi/web/simple',
# 'tsinghua': 'https://pypi.tuna.tsinghua.edu.cn/simple',
# 'netease': 'https://mirrors.163.com/pypi/simple/'}

# 当前系统环境变量PATH中第一个Python环境路径：
print(cur_py_path())
# C:\Python37\ (具体与个人系统环境变量PATH设置有关)

# 在常用安装位置查找Python目录：
print(all_py_paths())
# ['C:\\Python37\\',
# 'C:\\Anaconda3\\',
# 'C:\\Anaconda3\\envs\\py35\\',
# 'C:\\Anaconda3\\envs\\py36\\',
# 'C:\\Anaconda3\\envs\\py37\\',
# 'C:\\Anaconda3\\envs\\py38\\',
# 'C:\\Anaconda3\\envs\\py39\\']

# PyEnv类：
##########
# 实例化一个PyEnv类：
target = PyEnv("C:\\Python37\\")
# target = PyEnv() 与上面一行代码效果是一样的，省略参数时PyEnv内部会调用cur_py_path函数获取系统环境变量PATH中第一个Python目录路径，如果Python路径还未设置到PATH中，则创建一个不指向任何Python目录路径的PyEnv空实例，实例的所有方法被调用时都将返回空值(不同的方法返回的数据类型也不同，但都是可以判为False的空值，例如空列表、空字符串)

# 在该环境中安装模块(例如安装fastpip)：
print(target.install("fastpip", upgrade=1))
# 安装成功返回True，否则返回False。
# upgrade=1代表以升级模式安装最新版本，否则如果环境中已安装fastpip，install方法将不会再安装一次fastpip。
# 如果有需要，也可以增加以下关键字参数：
# index_url：str，例如index_url='https://mirrors.cloud.tencent.com/pypi/simple'。本次安装从镜像地址index_url处下载安装，否则从本地设置的全局镜像源地址安装，如果本地未设置全局镜像源地址，则从官方源PyPi安装，国内用户下载速度非常慢甚至可能因连接超时导致安装失败。
# timeout：int或float或None，例如timeout=30，表示设置安装超时限制30秒，如果安装用时超过30秒，则安装失败。None代表不设超时限制(默认)。
# no_tips：bool，例如no_tips=0，表示在安装过程中显示'正在安装xxx...'动态字样。
# no_output：bool，例如no_output=0，表示在安装结束后，在终端上输出安装时的安装信息。

# 待续...
